@@ -183,7 +183,7 @@ impl TestData {
             Header::CreateLink(self.link_add.clone()),
             fixt!(Signature),
         ));
-        self.scratch.add_header(header);
+        self.scratch.add_header(header, ChainTopOrdering::default());
     }
     fn delete_link(&self) {
         let op = DhtOpHashed::from_content_sync(DhtOp::RegisterRemoveLink(
@@ -201,7 +201,7 @@ impl TestData {
             Header::DeleteLink(self.link_remove.clone()),
             fixt!(Signature),
         ));
-        self.scratch.add_header(header);
+        self.scratch.add_header(header, ChainTopOrdering::default());
     }
     fn clear_scratch(&mut self) {
         self.scratch.drain_headers().for_each(|_| ());
